# vim: set ts=8 sw=4 sts=4 et ai tw=79:
"""
pstore-lib -- Python Protected Password Store (Library)
Copyright (C) 2012,2013,2015  Walter Doekes <wdoekes>, OSSO B.V.

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or (at
    your option) any later version.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
    USA.
"""


class PStoreException(Exception):
    class_ = 'general error'

    def __init__(self, *args, **kwargs):
        super(Exception, self).__init__(*args, **kwargs)
        if self.args or not self.description:
            self.args = tuple((self.class_,) + self.args)
        else:
            self.args = (self.class_, self.description)


class BackendError(PStoreException):
    class_ = 'backend error'
    description = 'undefined'


class BackendDown(BackendError):
    description = 'backend is down/unreachable'


class CryptError(PStoreException):
    class_ = 'crypto error'
    description = 'undefined'


class CryptBadPassword(CryptError):
    description = 'insufficient typing skills detected'


class CryptBadPubKey(CryptError):
    description = 'missing or invalid public key'


class CryptBadPrivKey(CryptError):
    description = 'missing or invalid private key'


class UserError(PStoreException):
    class_ = 'user error'
    description = 'undefined'


class NotAllowed(UserError):
    description = 'lookup denied (no permission or not found)'


class NotFound(UserError):
    description = 'lookup failed (not found)'


class NoNonce(UserError):
    description = ('nonce creation refused '
                   '(wrong user? too many unused nonces?)')



# vim: set ts=8 sw=4 sts=4 et ai tw=79:
"""
pstore-lib -- Python Protected Password Store (Library)
Copyright (C) 2012,2013,2015,2017  Walter Doekes <wdoekes>, OSSO B.V.

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or (at
    your option) any later version.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
    USA.
"""

# We only need to expose these. The gpg and sshrsa cryptography modules are
# imported by crypt.
__all__ = ('VERSION', 'VERSION_STRING', 'bytes', 'crypt', 'exceptions',
           'server')
# http://docs.python.org/2/tutorial/modules.html#importing-from-a-package

VERSION = (1, 2, 1)
VERSION_STRING = '1.2.1'



# vim: set ts=8 sw=4 sts=4 et ai tw=79:
"""
pstore-lib -- Python Protected Password Store (Library)
Copyright (C) 2013,2015,2017  Walter Doekes <wdoekes>, OSSO B.V.

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or (at
    your option) any later version.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
    USA.
"""
try:
    from pgpdump import AsciiData, BinaryData
except ImportError:
    from pstorelib.pgpdump_tiny import AsciiData, BinaryData


__all__ = ('get_pubkey_id_from_ascii', 'get_pubkey_id_from_binary')


def get_pubkey_id_from_ascii(data):
    generator = AsciiData(data)
    for packet in generator.packets():
        if packet.raw == 6:
            return packet.key_id
    return None


def get_pubkey_id_from_binary(data):
    generator = BinaryData(data)
    for packet in generator.packets():
        if packet.raw == 6:
            return packet.key_id
    return None


if __name__ == '__main__':
    from base64 import b64decode
    from unittest import TestCase, main

    class UnitTest(TestCase):
        HARM_PUBKEY_ASCII = '\n'.join((
            '-----BEGIN PGP PUBLIC KEY BLOCK-----',
            'Version: GnuPG v1.4.11 (GNU/Linux)',
            '',
            'mI0EULkrqAEEANNAbAZvH13iidylQmrm3EC1zCj8gm3gWsqK/0a8qKD9sDpjRX/c',
            'zbBzYdd5f1yzw2O1U9rAcnAFbAeBzsAcw2iDLVcnM6HP1F7Hyz1phR7IssmW4unw',
            'JYY75WIWjIvSK3gFcZMQNXWlfANs1nwZ+Z6UxaJDvPR7lPIb3ibUeLufABEBAAG0',
            'JUhhcm0gR2VlcnRzIChURVNUKSA8aGFybUBleGFtcGxlLmNvbT6IuAQTAQIAIgUC',
            'ULkrqAIbLwYLCQgHAwIGFQgCCQoLBBYCAwECHgECF4AACgkQt8MvZ2DlzsAD4AP/',
            'Tbu6Sc7IhmrlRdAN90BnxKNJDU9l8uWLGJ8dsli3pZ6NohdNubYgcwi5zBi3Cj8E',
            's0vYh8HBxkDPtAUI7vRyhAEw2Chwi1TWlOFEerpl5dNyxoHSDX2TQclnAkUw8KRv',
            'NLujHCK6p4mEjeBOZdn0r/Fs6YHkdN1y1VysnM3rr0C4jQRQuSuoAQQA0/RU/Er7',
            'ksyDndEcYOHOb6eBRGrbe+kIrbMWBRhgVN+FyAih+Zu9hACMFFof3OM2MVkQN8St',
            'vihPzryRZV7HVVJp0LplpzkUGDu5C4iTp1fGKsBZ23F1zfZyETEpMPYCWnKQeNzh',
            'VD6W7FBnSF5jVWy+Ro5oFc7w2cy7mibh0UUAEQEAAYkBPQQYAQIACQUCULkrqAIb',
            'LgCoCRC3wy9nYOXOwJ0gBBkBAgAGBQJQuSuoAAoJEN0HDbSvN/v/K2QD/3IW4Kxd',
            'bOgENnz+ov+aTRO948ooVxy7afdNK5lz41L9596rUSKJr2WFLaqlAQMf7KZTcv+V',
            'O9o+5UIHP5nOU8b2u0zV/FGdCIDSfc18iKOZmVmyCZCgG/JX01ZcianNPDMxu5tF',
            'ITbM+pPleA2LgAjOkRZhmX/ry7WZMNXGjNF0Y7UD/0reXaSJqA+gI0QoXSOYw5Sl',
            'LVs8T2Z40qp7FXhqf91OyhT/bwZHys9BudYZQzwA5a7a/NhyDmZFEk5FdCO7f6wl',
            'xy+EzYGZSKSPl9c0nHaL+ITKb+H65XmJbbxZ1AvzqqQH6k+0dUyJTzZn1qVLYYPP',
            'TeO36hkKrU3I+IJdE+GN',
            '=pxFX',
            '-----END PGP PUBLIC KEY BLOCK-----',
        ))

        HARM_PUBKEY_BINARY = b64decode(
            'mI0EULkrqAEEANNAbAZvH13iidylQmrm3EC1zCj8gm3gWsqK/0a8qKD9sDpjRX/c'
            'zbBzYdd5f1yzw2O1U9rAcnAFbAeBzsAcw2iDLVcnM6HP1F7Hyz1phR7IssmW4unw'
            'JYY75WIWjIvSK3gFcZMQNXWlfANs1nwZ+Z6UxaJDvPR7lPIb3ibUeLufABEBAAG0'
            'JUhhcm0gR2VlcnRzIChURVNUKSA8aGFybUBleGFtcGxlLmNvbT6IuAQTAQIAIgUC'
            'ULkrqAIbLwYLCQgHAwIGFQgCCQoLBBYCAwECHgECF4AACgkQt8MvZ2DlzsAD4AP/'
            'Tbu6Sc7IhmrlRdAN90BnxKNJDU9l8uWLGJ8dsli3pZ6NohdNubYgcwi5zBi3Cj8E'
            's0vYh8HBxkDPtAUI7vRyhAEw2Chwi1TWlOFEerpl5dNyxoHSDX2TQclnAkUw8KRv'
            'NLujHCK6p4mEjeBOZdn0r/Fs6YHkdN1y1VysnM3rr0C4jQRQuSuoAQQA0/RU/Er7'
            'ksyDndEcYOHOb6eBRGrbe+kIrbMWBRhgVN+FyAih+Zu9hACMFFof3OM2MVkQN8St'
            'vihPzryRZV7HVVJp0LplpzkUGDu5C4iTp1fGKsBZ23F1zfZyETEpMPYCWnKQeNzh'
            'VD6W7FBnSF5jVWy+Ro5oFc7w2cy7mibh0UUAEQEAAYkBPQQYAQIACQUCULkrqAIb'
            'LgCoCRC3wy9nYOXOwJ0gBBkBAgAGBQJQuSuoAAoJEN0HDbSvN/v/K2QD/3IW4Kxd'
            'bOgENnz+ov+aTRO948ooVxy7afdNK5lz41L9596rUSKJr2WFLaqlAQMf7KZTcv+V'
            'O9o+5UIHP5nOU8b2u0zV/FGdCIDSfc18iKOZmVmyCZCgG/JX01ZcianNPDMxu5tF'
            'ITbM+pPleA2LgAjOkRZhmX/ry7WZMNXGjNF0Y7UD/0reXaSJqA+gI0QoXSOYw5Sl'
            'LVs8T2Z40qp7FXhqf91OyhT/bwZHys9BudYZQzwA5a7a/NhyDmZFEk5FdCO7f6wl'
            'xy+EzYGZSKSPl9c0nHaL+ITKb+H65XmJbbxZ1AvzqqQH6k+0dUyJTzZn1qVLYYPP'
            'TeO36hkKrU3I+IJdE+GN'
        )

        def test_get_pubkey_id_from_ascii(self):
            value = get_pubkey_id_from_ascii(self.HARM_PUBKEY_ASCII)
            self.assertEqual(value, 'B7C32F6760E5CEC0')

        def test_get_pubkey_id_from_binary(self):
            value = get_pubkey_id_from_binary(self.HARM_PUBKEY_BINARY)
            self.assertEqual(value, 'B7C32F6760E5CEC0')

    main()



# vim: set ts=8 sw=4 sts=4 et ai tw=79:
"""
pstore-lib -- Python Protected Password Store (Library)
Copyright (C) 2013,2015,2017  Walter Doekes <wdoekes>, OSSO B.V.

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or (at
    your option) any later version.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
    USA.

The code herein is mostly taken from python-pgpdump by DanMcGee, which in turn
is derived from 'pgpdump' by Kazuhiko Yamamoto.

https://pypi.python.org/pypi/pgpdump/1.3
http://www.mew.org/~kazu/proj/pgpdump/

It is trimmed down to only return the PGP key id.
"""
import hashlib
from base64 import b64decode
from struct import unpack


__all__ = ('AsciiData', 'BinaryData')


def get_int(data, offset, size):
    letter = {1: 'B', 2: 'H', 4: 'I', 8: 'Q'}[size]
    binary = data[offset:(offset + size)]
    try:
        integer = unpack('>%s' % (letter,), binary)[0]
    except:
        raise Exception('unpack failed on %r' % (binary,))
    return integer


def my_ord(byte):
    if not isinstance(byte, int):
        byte = ord(byte)
    return byte


def get_mpi(data, offset):
    """
    Gets a multi-precision integer as per RFC-4880.
    Returns the MPI and the new offset.
    See: http://tools.ietf.org/html/rfc4880#section-3.2
    """
    # #return None, int((get_int(data, offset, 2) + 7) / 8) + 2
    mpi_len = get_int(data, offset, 2)
    offset += 2
    to_process = (mpi_len + 7) // 8
    mpi = 0
    i = -4
    for i in range(0, to_process - 3, 4):
        mpi <<= 32
        mpi += get_int(data, offset + i, 4)
    for j in range(i + 4, to_process):
        mpi <<= 8
        mpi += my_ord(data[offset + j])
    # Python 3.2 and later alternative:
    # #mpi = int.from_bytes(data[offset:offset + to_process], byteorder='big')
    offset += to_process
    return mpi, offset


class BinaryData(object):
    """
    The base object used for extracting PGP data packets. This expects fully
    binary data as input; such as that read from a .sig or .gpg file.
    """
    binary_tag_flag = 0x80

    def __init__(self, data):
        if not data:
            raise Exception("no data to parse")
        if len(data) <= 1:
            raise Exception("data too short")

        # 7th bit of the first byte must be a 1
        if not bool(my_ord(data[0]) & self.binary_tag_flag):
            raise Exception("incorrect binary data")
        self.data = data
        self.length = len(data)

    def packets(self):
        packet = self.get_public_key()
        if packet:
            yield packet

    def get_public_key(self):
        """
        Return the public key packet.
        A generator function returning PGP data packets.
        """
        offset = 0
        while offset < self.length:
            tag = my_ord(self.data[offset]) & 0x3f
            new = bool(my_ord(self.data[offset]) & 0x40)
            if new:
                pos = offset + 1
                data_offset, length, partial = new_tag_length(self.data, pos)
                data_offset += 1
            else:
                tag >>= 2
                data_offset, length = old_tag_length(self.data, offset)
            data_offset += 1
            offset += data_offset

            if tag == 6:
                end = offset + length
                packet_data = self.data[offset:end]
                packet = PublicKeyPacket(tag, new, packet_data)
                return packet

            offset += length
        return None


class AsciiData(BinaryData):
    def __init__(self, data):
        lines = [i.strip() for i in data.strip().split('\n')]

        if not lines[0].startswith('----'):  # -----BEGIN PGP PUBLIC KEY BLOCK
            raise ValueError('missing BEGIN PGP PUBLIC KEY BLOCK')
        lines.pop(0)

        if not lines[-1].startswith('----'): # -----END PGP PUBLIC KEY BLOCK
            raise ValueError('missing END PGP PUBLIC KEY BLOCK')
        lines.pop()

        # Drop optional checksum that starts with '='.
        if lines[-1].startswith('='):
            lines.pop()

        # Drop optional headers before the actual data.
        # E.g. "Version: GnuPG v1.4.11 (GNU/Linux)"
        if '' in lines:
            while lines[0] != '':
                lines.pop(0)
            lines.pop(0)
        # Should be only once.
        if '' in lines:
            raise ValueError('excess LF in public key block?')

        data = ''.join(i.strip() for i in lines)
        decoded = b64decode(data)
        super(AsciiData, self).__init__(decoded)


class PublicKeyPacket(object):
    def __init__(self, raw, new, data):
        self.raw = raw
        self.new = new
        self.length = len(data)
        self.data = data

        self.pubkey_version = None
        self.key_id = None
        self.raw_pub_algorithm = None
        self.pub_algorithm_type = None
        self.modulus = None
        self.exponent = None
        self.prime = None

        self.parse()

    def parse(self):
        self.pubkey_version = my_ord(self.data[0])
        offset = 1
        if self.pubkey_version in (2, 3):
            offset += 4  # raw_creation_time
            offset += 2  # days_valid

            self.raw_pub_algorithm = my_ord(self.data[offset])
            offset += 1
            offset = self.parse_key_material(offset)

            # #md5 = hashlib.md5()
            # Key type must be RSA for v2 and v3 public keys
            if self.pub_algorithm_type == "rsa":
                key_id = ('%X' % self.modulus)[-8:].zfill(8)
                self.key_id = key_id.encode('ascii')
                # #md5.update(get_int_bytes(self.modulus))
                # #md5.update(get_int_bytes(self.exponent))
            elif self.pub_algorithm_type == "elg":
                # Of course, there are ELG keys in the wild too. This formula
                # for calculating key_id and fingerprint is derived from an old
                # key and there is a test case based on it.
                key_id = ('%X' % self.prime)[-8:].zfill(8)
                self.key_id = key_id.encode('ascii')
                # #md5.update(get_int_bytes(self.prime))
                # #md5.update(get_int_bytes(self.group_gen))
            else:
                raise Exception(
                    "Invalid non-RSA v%d public key" % self.pubkey_version)
            # #self.fingerprint = md5.hexdigest().upper().encode('ascii')
        elif self.pubkey_version == 4:
            sha1 = hashlib.sha1()
            sha1.update(b'%c%c%c' % (
                0x99, (self.length >> 8) & 0xff, self.length & 0xff))
            sha1.update(self.data)
            fingerprint = sha1.hexdigest().upper().encode('ascii')
            self.key_id = fingerprint[24:]

            offset += 4  # raw_creation_time
            self.raw_pub_algorithm = my_ord(self.data[offset])
            offset += 1

            offset = self.parse_key_material(offset)
        else:
            raise Exception(
                "Unsupported public key packet, version %d" %
                self.pubkey_version)

        return offset

    def parse_key_material(self, offset):
        if self.raw_pub_algorithm in (1, 2, 3):
            self.pub_algorithm_type = "rsa"
            # n, e
            self.modulus, offset = get_mpi(self.data, offset)
            self.exponent, offset = get_mpi(self.data, offset)
        elif self.raw_pub_algorithm == 17:
            self.pub_algorithm_type = "dsa"
            # p, q, g, y
            self.prime, offset = get_mpi(self.data, offset)
            self.group_order, offset = get_mpi(self.data, offset)
            self.group_gen, offset = get_mpi(self.data, offset)
            self.key_value, offset = get_mpi(self.data, offset)
        elif self.raw_pub_algorithm in (16, 20):
            self.pub_algorithm_type = "elg"
            # p, g, y
            self.prime, offset = get_mpi(self.data, offset)
            self.group_gen, offset = get_mpi(self.data, offset)
            self.key_value, offset = get_mpi(self.data, offset)
        elif 100 <= self.raw_pub_algorithm <= 110:
            # Private/Experimental algorithms, just move on
            pass
        else:
            raise Exception(
                "Unsupported public key algorithm %d" % self.raw_pub_algorithm)

        return offset


def new_tag_length(data, start):
    """
    Takes a bytearray of data as input, as well as an offset of where to
    look. Returns a derived (offset, length, partial) tuple.
    """
    first = my_ord(data[start])
    offset = length = 0
    partial = False

    if first < 192:
        length = first
    elif first < 224:
        offset = 1
        length = ((first - 192) << 8) + my_ord(data[start + 1]) + 192
    elif first == 255:
        offset = 4
        length = get_int(data, start + 1, 4)
    else:
        # partial length, 224 <= l < 255
        length = 1 << (first & 0x1f)
        partial = True

    return (offset, length, partial)


def old_tag_length(data, start):
    """
    Takes a bytearray of data as input, as well as an offset of where to
    look. Returns a derived (offset, length) tuple.
    """
    offset = length = 0
    temp_len = my_ord(data[start]) & 0x03

    if temp_len in (0, 1, 2):
        size = (1, 2, 4)[temp_len]
        offset = size
        length = get_int(data, start + 1, size)
    elif temp_len == 3:
        length = len(data) - start - 1

    return (offset, length)


if __name__ == '__main__':
    from unittest import TestCase, main

    class UnitTest(TestCase):
        HARM_PUBKEY_ASCII = '\n'.join((
            '-----BEGIN PGP PUBLIC KEY BLOCK-----',
            'Version: GnuPG v1.4.11 (GNU/Linux)',
            '',
            'mI0EULkrqAEEANNAbAZvH13iidylQmrm3EC1zCj8gm3gWsqK/0a8qKD9sDpjRX/c',
            'zbBzYdd5f1yzw2O1U9rAcnAFbAeBzsAcw2iDLVcnM6HP1F7Hyz1phR7IssmW4unw',
            'JYY75WIWjIvSK3gFcZMQNXWlfANs1nwZ+Z6UxaJDvPR7lPIb3ibUeLufABEBAAG0',
            'JUhhcm0gR2VlcnRzIChURVNUKSA8aGFybUBleGFtcGxlLmNvbT6IuAQTAQIAIgUC',
            'ULkrqAIbLwYLCQgHAwIGFQgCCQoLBBYCAwECHgECF4AACgkQt8MvZ2DlzsAD4AP/',
            'Tbu6Sc7IhmrlRdAN90BnxKNJDU9l8uWLGJ8dsli3pZ6NohdNubYgcwi5zBi3Cj8E',
            's0vYh8HBxkDPtAUI7vRyhAEw2Chwi1TWlOFEerpl5dNyxoHSDX2TQclnAkUw8KRv',
            'NLujHCK6p4mEjeBOZdn0r/Fs6YHkdN1y1VysnM3rr0C4jQRQuSuoAQQA0/RU/Er7',
            'ksyDndEcYOHOb6eBRGrbe+kIrbMWBRhgVN+FyAih+Zu9hACMFFof3OM2MVkQN8St',
            'vihPzryRZV7HVVJp0LplpzkUGDu5C4iTp1fGKsBZ23F1zfZyETEpMPYCWnKQeNzh',
            'VD6W7FBnSF5jVWy+Ro5oFc7w2cy7mibh0UUAEQEAAYkBPQQYAQIACQUCULkrqAIb',
            'LgCoCRC3wy9nYOXOwJ0gBBkBAgAGBQJQuSuoAAoJEN0HDbSvN/v/K2QD/3IW4Kxd',
            'bOgENnz+ov+aTRO948ooVxy7afdNK5lz41L9596rUSKJr2WFLaqlAQMf7KZTcv+V',
            'O9o+5UIHP5nOU8b2u0zV/FGdCIDSfc18iKOZmVmyCZCgG/JX01ZcianNPDMxu5tF',
            'ITbM+pPleA2LgAjOkRZhmX/ry7WZMNXGjNF0Y7UD/0reXaSJqA+gI0QoXSOYw5Sl',
            'LVs8T2Z40qp7FXhqf91OyhT/bwZHys9BudYZQzwA5a7a/NhyDmZFEk5FdCO7f6wl',
            'xy+EzYGZSKSPl9c0nHaL+ITKb+H65XmJbbxZ1AvzqqQH6k+0dUyJTzZn1qVLYYPP',
            'TeO36hkKrU3I+IJdE+GN',
            '=pxFX',
            '-----END PGP PUBLIC KEY BLOCK-----',
        ))

        HARM_PUBKEY_BINARY = b64decode(
            'mI0EULkrqAEEANNAbAZvH13iidylQmrm3EC1zCj8gm3gWsqK/0a8qKD9sDpjRX/c'
            'zbBzYdd5f1yzw2O1U9rAcnAFbAeBzsAcw2iDLVcnM6HP1F7Hyz1phR7IssmW4unw'
            'JYY75WIWjIvSK3gFcZMQNXWlfANs1nwZ+Z6UxaJDvPR7lPIb3ibUeLufABEBAAG0'
            'JUhhcm0gR2VlcnRzIChURVNUKSA8aGFybUBleGFtcGxlLmNvbT6IuAQTAQIAIgUC'
            'ULkrqAIbLwYLCQgHAwIGFQgCCQoLBBYCAwECHgECF4AACgkQt8MvZ2DlzsAD4AP/'
            'Tbu6Sc7IhmrlRdAN90BnxKNJDU9l8uWLGJ8dsli3pZ6NohdNubYgcwi5zBi3Cj8E'
            's0vYh8HBxkDPtAUI7vRyhAEw2Chwi1TWlOFEerpl5dNyxoHSDX2TQclnAkUw8KRv'
            'NLujHCK6p4mEjeBOZdn0r/Fs6YHkdN1y1VysnM3rr0C4jQRQuSuoAQQA0/RU/Er7'
            'ksyDndEcYOHOb6eBRGrbe+kIrbMWBRhgVN+FyAih+Zu9hACMFFof3OM2MVkQN8St'
            'vihPzryRZV7HVVJp0LplpzkUGDu5C4iTp1fGKsBZ23F1zfZyETEpMPYCWnKQeNzh'
            'VD6W7FBnSF5jVWy+Ro5oFc7w2cy7mibh0UUAEQEAAYkBPQQYAQIACQUCULkrqAIb'
            'LgCoCRC3wy9nYOXOwJ0gBBkBAgAGBQJQuSuoAAoJEN0HDbSvN/v/K2QD/3IW4Kxd'
            'bOgENnz+ov+aTRO948ooVxy7afdNK5lz41L9596rUSKJr2WFLaqlAQMf7KZTcv+V'
            'O9o+5UIHP5nOU8b2u0zV/FGdCIDSfc18iKOZmVmyCZCgG/JX01ZcianNPDMxu5tF'
            'ITbM+pPleA2LgAjOkRZhmX/ry7WZMNXGjNF0Y7UD/0reXaSJqA+gI0QoXSOYw5Sl'
            'LVs8T2Z40qp7FXhqf91OyhT/bwZHys9BudYZQzwA5a7a/NhyDmZFEk5FdCO7f6wl'
            'xy+EzYGZSKSPl9c0nHaL+ITKb+H65XmJbbxZ1AvzqqQH6k+0dUyJTzZn1qVLYYPP'
            'TeO36hkKrU3I+IJdE+GN'
        )

        def get_pubkey_id_from_(self, data, class_):
            object = class_(data)
            for packet in object.packets():
                if packet.raw == 6:
                    return packet.key_id

        def test_get_pubkey_id_from_ascii(self):
            value = self.get_pubkey_id_from_(self.HARM_PUBKEY_ASCII,
                                             class_=AsciiData)
            self.assertEqual(value, 'B7C32F6760E5CEC0')

        def test_get_pubkey_id_from_binary(self):
            value = self.get_pubkey_id_from_(self.HARM_PUBKEY_BINARY,
                                             class_=BinaryData)
            self.assertEqual(value, 'B7C32F6760E5CEC0')

        def test_get_pubkey_id_from_upstream_ascii(self):
            try:
                from pgpdump import AsciiData as UpstreamAsciiData
            except ImportError:
                pass
            else:
                value = self.get_pubkey_id_from_(self.HARM_PUBKEY_ASCII,
                                                 class_=UpstreamAsciiData)
                self.assertEqual(value, 'B7C32F6760E5CEC0')

        def test_get_pubkey_id_from_upstream_binary(self):
            try:
                from pgpdump import BinaryData as UpstreamBinaryData
            except ImportError:
                pass
            else:
                value = self.get_pubkey_id_from_(self.HARM_PUBKEY_BINARY,
                                                 class_=UpstreamBinaryData)
                self.assertEqual(value, 'B7C32F6760E5CEC0')

    main()



# vim: set ts=8 sw=4 sts=4 et ai tw=79:
"""
pstore-lib -- Python Protected Password Store (Library)
Copyright (C) 2012,2013,2015  Walter Doekes <wdoekes>, OSSO B.V.

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or (at
    your option) any later version.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
    USA.
"""
try:
    from io import BytesIO
except ImportError:
    from StringIO import StringIO as BytesIO


__all__ = ('BytesIO', 'sendfile')


BytesIO  # touching it to make it look used


def can_seek(fp):
    """
    Some file-like objects do have a seek attribute but raise an Illegal Seek
    IOError on usage.
    """
    try:
        fp.seek(0, 1)  # SEEK_CUR, move 0 bytes forward/backward
    except:  # AttributeError, IOError
        return False
    return True


def get_size(fp):
    """
    Getting the size of a file-like object is sometimes possible. If it isn't,
    return -1.
    """
    if hasattr(fp, 'size'):
        size = fp.size()
    elif hasattr(fp, 'getvalue'):
        size = len(fp.getvalue())
    elif can_seek(fp):
        pos = fp.tell()
        fp.seek(0, 2)       # SEEK_END
        size = fp.tell()
        fp.seek(pos, 0)     # SEEK_SET
    else:
        size = -1
    return size


def sendfile(outfp, infp):
    """
    Transfer data between file objects.

    Similar to linux sendfile(2) which takes file descriptors and a few extra
    arguments. Returns the number of bytes written.

    At http://code.google.com/p/pysendfile there is a project which uses the
    actual syscall. But we won't be needing that.
    """
    bytes = 0
    bufsize = 65536

    while True:
        chunk = infp.read(bufsize)
        chunk_len = len(chunk)
        outfp.write(chunk)
        bytes += chunk_len
        if chunk_len != bufsize:
            break  # EOF

    return bytes


class FileWithoutTrailingEnter(object):
    """
    If you're inputting a file over stdin, you can consider trimming
    the trailing EOF. This file wrapper does that.
    """
    def __init__(self, fp):
        self.fp = fp
        self.buf = b''
        self.eof = False

    def read(self, size=-1):
        if size == -1:
            readsize = size
        else:
            readsize = size - len(self.buf) + 2  # read more bytes

        # More bytes are needed
        if not self.eof and (size == -1 or readsize > 0):
            data = self.fp.read(readsize)
            if size != -1 and len(data) < readsize:
                self.eof = True
            if data:
                self.buf = self.buf + data

        # No bytes to keep? Return quickly.
        if not self.buf.endswith(b'\n'):
            if size == -1:
                ret, self.buf = self.buf, ''
            else:
                ret, self.buf = self.buf[0:size], self.buf[size:]
            return ret

        # Ok, hold back one or two bytes.
        if self.buf.endswith(b'\r\n'):
            keep = 2
        else:
            keep = 1

        if size == -1:
            ret, self.buf = self.buf[0:-keep], self.buf[-keep:]
        elif size <= len(self.buf) - keep:
            ret, self.buf = self.buf[0:size], self.buf[size:]
        else:
            assert self.eof
            ret, self.buf = self.buf[0:-keep], self.buf[-keep:]

        return ret


if __name__ == '__main__':
    import os
    from sys import stdin
    from tempfile import TemporaryFile
    from unittest import TestCase, main

    class Test(TestCase):
        def test_can_seek_bytesio(self):
            self.assertTrue(can_seek(BytesIO()))

        def test_can_seek_file(self):
            with TemporaryFile() as t:
                self.assertTrue(can_seek(t))

        def test_can_seek_stdin(self):
            self.assertFalse(can_seek(stdin))

        def test_get_size_bytesio(self):
            self.assertEquals(get_size(BytesIO('12345')), 5)

        def test_get_size_file(self):
            with TemporaryFile() as t:
                t.write('12345')
                self.assertEquals(get_size(t), 5)
                t.write('1')
                self.assertEquals(get_size(t), 6)

        def test_get_size_stdin(self):
            self.assertEquals(get_size(stdin), -1)

    class TestFileWithoutTrailingEnter(TestCase):
        def create_child(self):
            rp, wp = os.pipe()

            childpid = os.fork()
            if not childpid:
                os.close(rp)
                self.wf = os.fdopen(wp, 'w')
                return True

            os.close(wp)
            self.rf = os.fdopen(rp, 'r')
            self.childpid = childpid
            return False

        def teardown_child(self):
            self.wf.close()
            os._exit(0)

        def teardown_parent(self):
            self.rf.close()
            os.waitpid(self.childpid, 0)

        def test_selftest(self):
            if self.create_child():
                self.wf.write('test\n')
                self.wf.flush()
                self.teardown_child()
            else:
                self.assertEquals(self.rf.read(5), 'test\n')
                self.teardown_parent()

        def test_trailing_nolf(self):
            if self.create_child():
                self.wf.write('ABC\nDEF')
                self.teardown_child()
            else:
                wrapped = FileWithoutTrailingEnter(self.rf)
                self.assertEquals(wrapped.read(4), 'ABC\n')
                self.assertEquals(wrapped.read(-1), 'DEF')
                self.teardown_parent()

        def test_trailing_nolf2(self):
            if self.create_child():
                self.wf.write('ABC\nDEF')
                self.teardown_child()
            else:
                wrapped = FileWithoutTrailingEnter(self.rf)
                self.assertEquals(wrapped.read(4), 'ABC\n')
                self.assertEquals(wrapped.read(3), 'DEF')
                self.assertEquals(wrapped.read(2), '')
                self.teardown_parent()

        def test_trailing_1lf(self):
            if self.create_child():
                self.wf.write('ABC\nDEF\n')
                self.teardown_child()
            else:
                wrapped = FileWithoutTrailingEnter(self.rf)
                self.assertEquals(wrapped.read(4), 'ABC\n')
                self.assertEquals(wrapped.read(-1), 'DEF')
                self.teardown_parent()

        def test_trailing_2crlf(self):
            if self.create_child():
                self.wf.write('ABC\n\r\n\r\n')
                self.teardown_child()
            else:
                wrapped = FileWithoutTrailingEnter(self.rf)
                self.assertEquals(wrapped.read(32), 'ABC\n\r\n')
                self.teardown_parent()

        def test_trailing_2lf(self):
            if self.create_child():
                self.wf.write('ABC')
                self.wf.flush()
                self.wf.write('DEF\n')
                self.wf.flush()
                self.wf.write('\n\n')
                self.wf.flush()
                self.teardown_child()
            else:
                wrapped = FileWithoutTrailingEnter(self.rf)
                self.assertEquals(wrapped.read(4), 'ABCD')
                self.assertEquals(wrapped.read(3), 'EF\n')
                self.assertEquals(wrapped.read(2), '\n')
                self.teardown_parent()

    main()  # unittest.main()



# vim: set ts=8 sw=4 sts=4 et ai tw=79:
# BEWARE: this is not py3-safe: should be removed..
"""
pstore-lib -- Python Protected Password Store (Library)
Copyright (C) 2010,2012,2013,2015  Walter Doekes <wdoekes>, OSSO B.V.

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or (at
    your option) any later version.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
    USA.

This module implements a couple of functions to ease reading of OpenSSH public
and private keys and using those keys to encrypt and decrypt short messages.

Note that decrypting encrypted private SSH keys is only implemented for
DES-EDE3-CBC.

The class OAEP and helper function make_mgf1 are taken from PyCrypto
(python-crypto) bug report 328027 (launchpad/pycrypto). The author of that code
is Ryan Kelly (~ryan-rfk).

Dependencies:
* PyCrypto (python-crypto)
* ASN.1 tools for python (python-pyasn1)
"""
from __future__ import absolute_import

from base64 import b16decode, b64decode, b64encode
from hashlib import md5, sha1       # md5 for DES-EDE3-CBC, sha1 for OAEP
from math import ceil
from os import urandom

try:
    import Crypto
    Crypto
except ImportError as e:
    raise ImportError(e.args[0] + '\n\n*HINT* apt-get install python-crypto')

from Crypto.Cipher import DES3      # decrypt DES-EDE3-CBC private key
from Crypto.PublicKey import RSA    # en-/decrypt data stream
from Crypto.Util.number import bytes_to_long, long_to_bytes

try:
    from Crypto.Hash.HMAC import _strxor as strxor
except ImportError:
    try:
        from Crypto.Util.strxor import strxor
    except ImportError:
        from Crypto.Util import strxor

try:
    import pyasn1
except ImportError as e:
    raise ImportError(e.args[0] + '\n\n*HINT* apt-get install python-pyasn1')

try:
    from pyasn1.codec.ber import decoder as asn1decoder
except ImportError:
    from pyasn1.v1.codec.ber import decoder as asn1decoder

from pstorelib.exceptions import (CryptError, CryptBadPassword,
                                  CryptBadPrivKey, CryptBadPubKey)


# Only exposing RSACrypt and SSHKeyParser
__all__ = ('RSACrypt', 'SSHKeyParser')


class SSHKeyParser(object):
    """
    A simple interface to parse SSH public and private keys.
    """
    @staticmethod
    def default_password_cb():
        from getpass import getpass
        return (getpass('Enter passphrase for RSA key: '), True)

    def __init__(self):
        """
        Construct an SSHKeyParser.
        """
        self.password_cb = self.default_password_cb

    def set_password_cb(self, password_cb):
        """
        Set a password callback. Should return (password, allow_caching).
        """
        self.password_cb = password_cb

    def parse_public(self, line):
        """
        Parse a public key line in the ssh-rsa form: "ssh-rsa AAA..."

        Returns a dictionary with RSA variables e and n.
        """
        items = line.split(' ', 2)
        if len(items) == 3:
            type, blob, comment = items
        elif len(items) == 2:
            type, blob = items
        else:
            raise CryptBadPubKey('split failed', line)

        if type != 'ssh-rsa':
            raise CryptBadPubKey('expected type ssh-rsa', type)
        try:
            blob = b64decode(blob)
        except:
            raise CryptBadPubKey('base64 decode failed', blob)

        results = []
        while blob != '':
            length = [long(ord(i)) for i in blob[0:4]]
            length = (length[0] << 24 | length[1] << 16 | length[2] << 8 |
                      length[3])
            part = blob[4:(4 + length)]
            blob = blob[(4 + length):]
            if len(part) != length:
                raise CryptBadPubKey(('elongation failed: expected %d for %d, '
                                      'got %d') %
                                     (length, len(results) + 1, len(part)))
            results.append(part)

        if len(results) != 3:
            raise CryptBadPubKey('expected 3 items in blob, got %d' %
                                 (len(results),), results)
        if results[0] != 'ssh-rsa':
            raise CryptBadPubKey('expected type ssh-rsa in blob, got %s' %
                                 (results[0],), results)

        type = results[0]
        rsa_e = bytes_to_long(results[1])
        rsa_n = bytes_to_long(results[2])

        return {'type': type, 'n': rsa_n, 'e': rsa_e}

    def parse_private(self, text):
        """
        Parse a private key file in the ssh-rsa form: --BEGIN RSA PRIVATE...

        Returns a dictionary with RSA variables n, e, d, p and q.
        """
        lines = text.split('\n')
        del text
        # Strip all from begin until marker.
        while lines[0].strip() != '-----BEGIN RSA PRIVATE KEY-----':
            lines.pop(0)
        lines.pop(0)
        # Strip all from end back to marker.
        while lines[-1].strip() != '-----END RSA PRIVATE KEY-----':
            lines.pop()
        lines.pop()

        # Get base64 data from end and decode to binary.
        data = []
        while len(lines) and lines[-1].strip() != '':
            data.insert(0, lines[-1].strip())
            lines.pop()
        blob = ''.join(data)
        del data

        try:
            blob = b64decode(blob)
        except:
            raise CryptBadPrivKey('base64 decode failed', blob)

        # If there are lines left, they contain encryption info.
        if len(lines):
            if (len(lines) != 3 or
                    lines[2].strip() != '' or
                    not lines[0].startswith('Proc-Type: 4,ENCRYPTED') or
                    not lines[1].startswith('DEK-Info: ')):
                raise CryptBadPrivKey('encryption info parse fail', blob)

            dek_info = lines[1][9:].strip().split(',')
            if len(dek_info) != 2:
                raise CryptBadPrivKey('expected comma-separated DEK-Info',
                                      lines[1])
        else:
            dek_info = None
        del lines

        # ASN.1-decode the blob and get the values. Do it once or three times
        # depending on whether we need to ask for a password.
        for i in range((1, 3)[bool(dek_info)]):
            if dek_info:
                key = self.decrypt(blob, dek_info)
            else:
                key = blob

            if not dek_info and (not key or key[0] != '0'):
                raise CryptBadPrivKey('expected ASN.1 to begin with 0', key)

            try:
                items = asn1decoder.decode(key)
            except pyasn1.error.PyAsn1Error as e:
                if not dek_info:
                    raise CryptBadPrivKey('ASN.1 decode failed',
                                          (': '.join(str(i) for i in e.args)))
            else:
                break
        else:
            # We iterated over dek_info three times while calling decrypt
            # (asking for a password). Still nothing: bad password (or invalid
            # data).
            assert dek_info
            raise CryptBadPassword()

        if long(items[0][0]) != 0:
            raise CryptBadPrivKey('expected pub key version 0', items[0][0])

        type = 'ssh-rsa'
        rsa_n, rsa_e = long(items[0][1]), long(items[0][2])
        rsa_d = long(items[0][3])
        rsa_p, rsa_q = long(items[0][4]), long(items[0][5])

        return {'type': type,               # "ssh-rsa"
                'n': rsa_n, 'e': rsa_e,     # (p * q), exponent
                'd': rsa_d,                 # private key exponent
                'p': rsa_p, 'q': rsa_q}     # two private primes

    def decrypt(self, data, dek_info):
        """
        Decrypt password encrypted private key data (the encrypted ASN.1
        private key data).
        """
        if dek_info[0] != 'DES-EDE3-CBC':
            raise NotImplementedError(('unexpected algorithm %s, expected '
                                       'DES-EDE3-CBC') % (dek_info[0],))

        # Convert hex to binary.
        salt_and_iv = b16decode(dek_info[1])

        # Get password from user.
        password, allow_caching = self.password_cb()
        del allow_caching

        # Convert password to 24 bytes for 3DES.
        des_password = ''
        while len(des_password) < 24:
            tmp = ''
            if len(des_password):
                tmp = des_password
            tmp += password + salt_and_iv
            des_password += md5(tmp).digest()
        des_password = des_password[0:24]

        # Create a 3DES decryptor with the password and initialization vector
        # and decrypt data.
        return DES3.new(des_password, DES3.MODE_CBC, salt_and_iv).decrypt(data)


def make_mgf1(hash):
    """
    Make an MGF1 function using the given hash function.

    Given a hash function implementing the standard hash function interface,
    this function returns a Mask Generation Function using that hash.
    """
    def mgf1(mgfSeed, maskLen):
        """
        Mask Generation Function based on a hash function.

        Given a seed byte string 'mgfSeed', this function will generate
        and return a mask byte string  of length 'maskLen' in a manner
        approximating a Random Oracle.

        The algorithm is from PKCS#1 version 2.1, appendix B.2.1.
        """
        hLen = hash().digest_size
        if maskLen > (2 ** 32) * hLen:
            raise NotImplementedError('mask too long')
        T = ''
        for counter in range(int(ceil(maskLen / float(hLen)))):
            C = long_to_bytes(counter)
            C = ('\x00' * (4 - len(C))) + C
            assert len(C) == 4, 'counter was too big'
            T += hash(mgfSeed + C).digest()
        assert len(T) >= maskLen, 'generated mask was too short'
        return T[:maskLen]
    return mgf1


MGF1_SHA1 = make_mgf1(sha1)


class OAEP(object):
    """
    Class implementing OAEP encoding/decoding.

    This class can be used to encode/decode byte strings using the Optimal
    Asymmetic Encryption Padding Scheme.  It requires a source of random bytes,
    a hash function and a mask generation function.  By default SHA-1 is used
    as the hash function, and MGF1-SHA1 is used as the mask generation
    function.

    The method 'encode' will encode a byte string using this padding scheme,
    and the complementary method 'decode' will decode it.

    The algorithms are from PKCS#1 version 2.1, section 7.1.
    """
    def __init__(self, randbytes=urandom, hash=sha1, mgf=MGF1_SHA1):
        self.randbytes = randbytes
        self.hash = hash
        self.mgf = mgf

    def encode(self, k, M, L=''):
        """
        Encode a message using OAEP.

        This method encodes a byte string 'M' using Optimal Asymmetric
        Encryption Padding.  The argument 'k' must be the size of the private
        key modulus in bytes.  If specified, 'L' is a label for the
        encoding.
        """
        # Calculate label hash, unless it is too long.
        if L:
            limit = getattr(self.hash, 'input_limit', None)
            if limit and len(L) > limit:
                raise CryptError('encrypt label too long')
        lHash = self.hash(L).digest()
        # Check length of message against size of key modulus
        mLen = len(M)
        hLen = len(lHash)
        if mLen > k - 2 * hLen - 2:
            raise CryptError('encrypt message too long')
        # Perform the encoding.
        PS = '\x00' * (k - mLen - 2 * hLen - 2)
        DB = lHash + PS + '\x01' + M
        assert len(DB) == k - hLen - 1, 'DB length is incorrect'
        seed = self.randbytes(hLen)
        dbMask = self.mgf(seed, k - hLen - 1)
        maskedDB = strxor(DB, dbMask)
        seedMask = self.mgf(maskedDB, hLen)
        maskedSeed = strxor(seed, seedMask)
        return '\x00' + maskedSeed + maskedDB

    def decode(self, k, EM, L=''):
        """
        Decode a message using OAEP.

        This method decodes a byte string 'EM' using Optimal Asymmetric
        Encryption Padding.  The argument 'k' must be the size of the private
        key modulus in bytes.  If specified, 'L' is the label used for the
        encoding.
        """
        # Generate label hash, for sanity checking.
        lHash = self.hash(L).digest()
        hLen = len(lHash)
        # Split the encoded message.
        Y = EM[0]
        maskedSeed = EM[1:(hLen + 1)]
        maskedDB = EM[(hLen + 1):]
        # Perform the decoding.
        seedMask = self.mgf(maskedDB, hLen)
        seed = strxor(maskedSeed, seedMask)
        dbMask = self.mgf(seed, k - hLen - 1)
        DB = strxor(maskedDB, dbMask)
        # Split the DB string.
        lHash1 = DB[:hLen]
        x01pos = hLen
        while x01pos < len(DB) and DB[x01pos] != '\x01':
            x01pos += 1
        M = DB[(x01pos + 1):]
        # All sanity-checking done at end, to avoid timing attacks.
        valid = True
        if x01pos == len(DB):  # No \x01 byte
            valid = False
        if lHash1 != lHash:    # Mismatched label hash
            valid = False
        if Y != '\x00':        # Invalid leading byte
            valid = False
        if not valid:
            raise CryptError('decrypt de-padding failed validation')
        return M


class RSACrypt(object):
    """
    Wrapper around RSA encrypting/decrypting.

    It uses "default" OAEP padding for the data.
    """
    def __init__(self, n, e, d=None, p=None, q=None):
        # From: https://www.dlitz.net/software/pycrypto/api/current/ _
        #         Crypto.PublicKey.RSA.RSAImplementation-class.html#construct
        #
        # Construct an RSA key object from a tuple of valid RSA components.
        #
        # The modulus n must be the product of two primes. The public exponent
        # e must be odd and larger than 1.
        #
        # In case of a private key, the following equations must apply:
        # - e != 1
        # - p*q = n
        # - e*d = 1 mod (p-1)(q-1)
        # - u = inverse of p mod q.
        # - p*u = 1 mod q
        #
        # Parameters:
        # - n = RSA modulus
        # - e = Public exponent
        # - d = Private exponent (only required if the key is private)
        # - p = First factor of n (optional)
        # - q = Second factor of n (optional)
        #
        if d:
            self.rsa = RSA.construct((n, e, d, p, q))
        else:
            self.rsa = RSA.construct((n, e))

        # Get modulus length for encoder.
        self.padlen = len(long_to_bytes(n))
        # The padder is optional, but preferable. It:
        # (1) Adds an element of randomness which can be used to convert a
        # deterministic encryption scheme (e.g., traditional RSA) into a
        # probabilistic scheme.
        # (2) Prevents partial decryption of ciphertexts (or other information
        # leakage) by ensuring that an adversary cannot recover any portion of
        # the plaintext without being able to invert the trapdoor one-way
        # permutation .
        self.padder = OAEP()

    def encrypt(self, data):
        """
        Encrypt the data.
        """
        if self.padder:
            data = self.padder.encode(self.padlen, data)
            assert data[0] == '\x00', 'Expected leading 0-byte from OAEP'
        results = self.rsa.encrypt(data, '')
        assert len(results) == 1
        return results[0]

    def decrypt(self, data):
        """
        Decrypt the data.
        """
        result = self.rsa.decrypt((data,))

        if self.padder:
            # Must pad data back to padlen (see assertion above). Note that
            # unencoded binary data suffers from the same problem.
            result = self.padder.decode(self.padlen,
                                        result.rjust(self.padlen, '\x00'))
        return result


if __name__ == '__main__':
    import unittest

    class Test(unittest.TestCase):
        maxDiff = None
        unencrypted_key = (
            '''-----BEGIN RSA PRIVATE KEY-----
            MIICWwIBAAKBgQDQhys3JAt1XIKClC0x+//SqdEpa3BkYvZF8vOddDngS6gibhvD
            mWkJ58z1i1917TTj4Hj0bbkK81hJ2JFMdROJhtffMOtmO6ibsy/GFJ+h170euVgW
            cLsVhj6o6hYJkvNaODXoC2NU4O26k3uiYb/iN7/R3tq6ojDloYQjhHpMiwIBIwKB
            gQCa6BF5azu2RLi4xdhuRivBEG992siFFlCLu8rp/pFkynzmYGxlaqXM2BSZJbSg
            vtbVKmh7D64IInwZmZCB8JIqTrj0s3qMzjNHG93Qwp5tEvQaCiBysrg8if5/n0ni
            FiaOXFEbYFc0b2FrOPo0MbWrbF6oTIcuryoNjzARASUx+wJBAOgtLwmsefLParMY
            ZnwToRG4CUTWOl5lqnSg3er29YNTw/fiTe5gSaRbhCfm90gYiBHOa2gliTHcyk5S
            WkWwIu0CQQDl7Mk1o8xG9qALzzy58dBnlRnJVi36XWhX2uQe2SijCTkS0rovA85/
            QE13fu6vWi52hatKxCPTTLpNhMYMBGZXAkEAsxuKr7DwXDmbdDdlAKFtn/Re6/W3
            /62Dfoq50oP/OWyBO5FSD6ldYYhtQ1pm/Ro9FQzzxV7I6/N3fj+Hd5aBVwJATtTX
            RZc+uT6dReCnGy5WFOKpwV9gOJUN1PqXWwigN+Xnrq6mPAFOHQAaj164PB7rW9YO
            1844KzA/4BBD5tzv4wJAOEQodXRqV2nAB4PvvKJSqgf5V33MUs0ihRPcDp7cJpNJ
            U51Ole+usiLLZCWLgaW8Excsr+UGicdOQTIAxm4OPQ==
            -----END RSA PRIVATE KEY-----'''.replace('  ', ''),
            '''ssh-rsa\x20
            AAAAB3NzaC1yc2EAAAABIwAAAIEA0IcrNyQLdVyCgpQtMfv/0qnRKWtwZGL2RfLz
            nXQ54EuoIm4bw5lpCefM9Ytfde004+B49G25CvNYSdiRTHUTiYbX3zDrZjuom7Mv
            xhSfode9HrlYFnC7FYY+qOoWCZLzWjg16AtjVODtupN7omG/4je/0d7auqIw5aGE
            I4R6TIs= abc@example.com'''.replace('  ', '').replace('\n', '')
        )
        encrypted_key = (
            '''-----BEGIN RSA PRIVATE KEY-----
            Proc-Type: 4,ENCRYPTED
            DEK-Info: DES-EDE3-CBC,9C514622E5E171E5

            KkqpzaQRbUfI9gFORshyZFvMy6oF/qEG69ctqXSl09PrWaQDMF5FVFRR2Yq5Rzi/
            mIB4EhgTAM55bWgtD3o2kDpLXfIWauvuu20W37NH87Y4JgDI+No457THmESfRCF8
            +HpYgl5yM19cy90zSUGzGpZmkSK9sg4xi+Yy6RNhUbsS9x1QLAjV1dsDYWXroCGZ
            6q7S3Nl6W0KwS5r8wOEnG+gkGb9i6wvjcG7VfMVDHleVw6fdeSy725iuLPtw3h71
            eavBy0WuUQAYv9yOMCa4IilU7c7dcGRU4g521ayydTbku9Kv+Ck/J6aug9MV84yq
            Irt++O0361ZtklrE1CdhdPCDzfEis4Bg8yjJnLO94AlT8mfjT1UyZFlbI2eYU+5o
            EF6GGU/dwxaD7OUeM2bbjZnYr6rCXA0iy8rBckIMHAUrC60u63ExhpB56C6ep6if
            dAZAbPhyQ+yRO6hbcJZglwUs9a627WFvyyv91gU5Nt67DoJni6DsjI2A44p0wcPm
            zZ26HDH2JAWB7GTFVELWWFP2lGJPBIjOQfN0n/Ktu/IPRScKWdZtYoyJAK1JHh28
            s6TW6X71a1vdv9Taf+/1c5ftXTi6sLHIQbgQHdwwBK2RLs9m2eNwqMYN9LQBkC9E
            bhx3Mv1V9WlAQsU5h0p0S/xUU1dKJYyFtQ7WOQitEl3cyNCQeuFFuljb8fV7Aj9U
            9hafrrX4YNByGegWA4d6mvSEqSJVX5QFZjLjeG18OKzs/ZwLAKsVc7Jbf1GmC9bn
            gg91NY26as/VkQo5YaBIfsOEPwOVK9PavLy5maJQdzc=
            -----END RSA PRIVATE KEY-----'''.replace('  ', ''),
            '''ssh-rsa\x20
            AAAAB3NzaC1yc2EAAAABIwAAAIEAqxSM3t3iX4OSBBgfXnFmFjQ0kYiHpQrgKfPb
            d7BuMVeT6BGp/+4l2ZIq9xEQF4J5mdEHuyAJrHdFRq7if5+OiGUuGOVEnAfzWjHg
            vErranLZo+6XRMsnvMG6DiD/Ys+uySw7tFw3mxzDgNxYbXaTc/Bbnqy6hi7/JSfX
            AqjSqKE= def@example.com'''.replace('  ', '').replace('\n', '')
        )

        def bignum(self, value):
            return bytes_to_long(b64decode(value))

        def test_bignum(self):
            input = 1234567890123456789012345678901234567890L
            text = b64encode(long_to_bytes(input))
            output = self.bignum(text)
            self.assertEqual(output, input)

        def test_OAEP_1(self):
            oaep = OAEP()
            data = 'Hello world!'
            enc1 = oaep.encode(128, data)
            enc2 = oaep.encode(128, data)
            # The random generator in OAEP should ensure that enc1 != enc2
            self.assertNotEqual(enc1, enc2)
            # But the unencoded contents should be the same.
            dec1 = oaep.decode(128, enc1)
            dec2 = oaep.decode(128, enc2)
            self.assertEqual(dec1, dec2)
            self.assertEqual(dec2, data)

        def test_OAEP_2(self):
            input = ('AMoc98M5otEvaSpqxhxw873WHo0R7HLmQuSMnhZDGRASDmCnjk+O82wW'
                     'GJiBAZH63rnzEDs0RMCbBhfd+9RdESVK1qhYq98T/vJDvWRT/qgvqHg3'
                     'aigkHQ9tcCmF3qDGRgxiTpeDwm+I42HYa3M+jZtKvHKX68qlYSd//Wnp'
                     'lKk=')
            enc = b64decode(input)
            oaep = OAEP()
            dec = oaep.decode(128, enc)
            self.assertEqual(dec, 'Hello world!')

        def test_parse_public_ssh_key_1(self):
            parser = SSHKeyParser()
            ret = parser.parse_public(self.unencrypted_key[1])
            self.assertEqual(ret, {
                'type': 'ssh-rsa',
                'n': self.bignum('''
                    0IcrNyQLdVyCgpQtMfv/0qnRKWtwZGL2RfLznXQ54EuoIm4bw5lpCefM9Y
                    tfde004+B49G25CvNYSdiRTHUTiYbX3zDrZjuom7MvxhSfode9HrlYFnC7
                    FYY+qOoWCZLzWjg16AtjVODtupN7omG/4je/0d7auqIw5aGEI4R6TIs=
                '''),
                'e': 35L,
            })

        def test_parse_public_ssh_key_2(self):
            parser = SSHKeyParser()
            ret = parser.parse_public(self.encrypted_key[1])
            self.assertEqual(ret, {
                'type': 'ssh-rsa',
                'n': self.bignum('''
                    qxSM3t3iX4OSBBgfXnFmFjQ0kYiHpQrgKfPbd7BuMVeT6BGp/+4l2ZIq9xE
                    QF4J5mdEHuyAJrHdFRq7if5+OiGUuGOVEnAfzWjHgvErranLZo+6XRMsnvM
                    G6DiD/Ys+uySw7tFw3mxzDgNxYbXaTc/Bbnqy6hi7/JSfXAqjSqKE=
                '''),
                'e': 35L,
            })

        def test_parse_private_ssh_key_1(self):
            parser = SSHKeyParser()
            ret = parser.parse_private(self.unencrypted_key[0])
            self.assertEqual(ret, {
                'type': 'ssh-rsa',
                'n': self.bignum('''
                    0IcrNyQLdVyCgpQtMfv/0qnRKWtwZGL2RfLznXQ54EuoIm4bw5lpCefM9Yt
                    fde004+B49G25CvNYSdiRTHUTiYbX3zDrZjuom7MvxhSfode9HrlYFnC7FY
                    Y+qOoWCZLzWjg16AtjVODtupN7omG/4je/0d7auqIw5aGEI4R6TIs=
                '''),
                'e': 35L,
                'd': self.bignum('''
                    mugReWs7tkS4uMXYbkYrwRBvfdrIhRZQi7vK6f6RZMp85mBsZWqlzNgUmSW
                    0oL7W1Spoew+uCCJ8GZmQgfCSKk649LN6jM4zRxvd0MKebRL0GgogcrK4PI
                    n+f59J4hYmjlxRG2BXNG9hazj6NDG1q2xeqEyHLq8qDY8wEQElMfs=
                '''),
                'p': self.bignum('''
                    6C0vCax58s9qsxhmfBOhEbgJRNY6XmWqdKDd6vb1g1PD9+JN7mBJpFuEJ+b
                    3SBiIEc5raCWJMdzKTlJaRbAi7Q==
                '''),
                'q': self.bignum('''
                    5ezJNaPMRvagC888ufHQZ5UZyVYt+l1oV9rkHtkoowk5EtK6LwPOf0BNd37
                    ur1oudoWrSsQj00y6TYTGDARmVw==
                '''),
            })

        def test_parse_private_ssh_key_2(self):
            parser = SSHKeyParser()
            parser.set_password_cb((lambda: ('testtest', False)))
            ret = parser.parse_private(self.encrypted_key[0])
            self.assertEqual(ret, {
                'type': 'ssh-rsa',
                'n': self.bignum('''
                    qxSM3t3iX4OSBBgfXnFmFjQ0kYiHpQrgKfPbd7BuMVeT6BGp/+4l2ZIq9xE
                    QF4J5mdEHuyAJrHdFRq7if5+OiGUuGOVEnAfzWjHgvErranLZo+6XRMsnvM
                    G6DiD/Ys+uySw7tFw3mxzDgNxYbXaTc/Bbnqy6hi7/JSfXAqjSqKE=
                '''),
                'e': 35L,
                'd': self.bignum('''
                    l4c68Ugu5uI/f/gbyK2U7xhL0VurV6qMB+ad0GkRJGODAL8wK9LuUv3OSIt
                    0pxvKyhDplxxgVu1L/MbInOxoS3kADYKzbyga0JI3klJmQ9R1yqZcIspjuC
                    sbT7Nu5M0CYo3qiW+o/aLpzdg1BMc3pDDeJ1T3m5aNubzpZv3TGfs=
                '''),
                'p': self.bignum('''
                    1/pyyGEfYSV3RN7j94GRJVGlwuRqUYwLGgD2pmIyJXJ6CHml1KzZAFv+mLn
                    dUt90gSsTDWR/SLv6gok+vgflzQ==
                '''),
                'q': self.bignum('''
                    ysg8BTGskkyD1E99ckC2AtF0NhEFxqb6f0/vCt3sRucJKLD/vybNCaGZ2+b
                    VETte1OtneTY2NAZAKtrkSBY6JQ==
                '''),
            })

        def test_rsawrapper(self):
            data = 'Hello world!'
            crypt = RSACrypt(
                n=self.bignum('''
                    qxSM3t3iX4OSBBgfXnFmFjQ0kYiHpQrgKfPbd7BuMVeT6BGp/+4l2ZIq9xE
                    QF4J5mdEHuyAJrHdFRq7if5+OiGUuGOVEnAfzWjHgvErranLZo+6XRMsnvM
                    G6DiD/Ys+uySw7tFw3mxzDgNxYbXaTc/Bbnqy6hi7/JSfXAqjSqKE=
                '''),
                e=35L
            )

            # With the OAEP() padder the output is in (127, 128). Without it,
            # it's 128.
            encrypted = crypt.encrypt(data)
            self.assertTrue(len(encrypted) in (127, 128))

            crypt = RSACrypt(
                n=self.bignum('''
                    qxSM3t3iX4OSBBgfXnFmFjQ0kYiHpQrgKfPbd7BuMVeT6BGp/+4l2ZIq9xE
                    QF4J5mdEHuyAJrHdFRq7if5+OiGUuGOVEnAfzWjHgvErranLZo+6XRMsnvM
                    G6DiD/Ys+uySw7tFw3mxzDgNxYbXaTc/Bbnqy6hi7/JSfXAqjSqKE=
                '''),
                e=35L,
                d=self.bignum('''
                    l4c68Ugu5uI/f/gbyK2U7xhL0VurV6qMB+ad0GkRJGODAL8wK9LuUv3OSIt
                    0pxvKyhDplxxgVu1L/MbInOxoS3kADYKzbyga0JI3klJmQ9R1yqZcIspjuC
                    sbT7Nu5M0CYo3qiW+o/aLpzdg1BMc3pDDeJ1T3m5aNubzpZv3TGfs=
                '''),
                p=self.bignum('''
                    1/pyyGEfYSV3RN7j94GRJVGlwuRqUYwLGgD2pmIyJXJ6CHml1KzZAFv+mLn
                    dUt90gSsTDWR/SLv6gok+vgflzQ==
                '''),
                q=self.bignum('''
                    ysg8BTGskkyD1E99ckC2AtF0NhEFxqb6f0/vCt3sRucJKLD/vybNCaGZ2+b
                    VETte1OtneTY2NAZAKtrkSBY6JQ==
                ''')
            )
            decrypted = crypt.decrypt(encrypted)
            self.assertEqual(decrypted, data)

    unittest.main()



#!/usr/bin/env python
# vim: set ts=8 sw=4 sts=4 et ai tw=79:
"""
pstore setup -- Python Protected Password Store (setup)
Copyright (C) 2012,2013,2015  Walter Doekes <wdoekes>, OSSO B.V.

This script is called setups.py and not setup.py. It contains two separate
setup calls, one for `pstore` and one for `django-pstore`.

During the `make dist` call, this file is copied over setup.py and the
appropriate call is appended. That way both packages get their own setup.py.
"""
import re

# TODO: comments about this...
# SAFELY IGNORE THIS WARNING. THE REQUIREMENTS GET PACKAGED!
# /usr/lib/python2.7/distutils/dist.py:267: UserWarning: Unknown distribution
# option: 'install_requires'
from distutils.core import setup
from distutils.version import LooseVersion  # strict is too ~, even for me
from distutils.command.build_py import build_py
_find_package_modules_orig = build_py.find_package_modules

# TODO: see this
# See also, for metadata help:
# http://docs.python.org/2/distutils/setupscript.html#additional-meta-data
# See also, for deb/ubuntu packaging instructions:
# http://ubuntuforums.org/showthread.php?t=1002909

# TODO: add readme/manpage?

try:
    # When installing the django-pstore package before the pstore package, this
    # would fail.
    from pstorelib import VERSION_STRING
except ImportError:
    VERSION_STRING = None


with open('CHANGES.md') as file:
    matcher = re.compile(r'^[0-9?]{4}-[0-9?]{2}-[0-9?]{2}: ([0-9].*?)\s*$')
    matches = []
    for line in file:
        match = matcher.match(line)
        if match:
            matches.append(LooseVersion(match.groups()[0]))
    # Double check that versions are in the right order
    for i in range(1, len(matches)):
        high = matches[i - 1]
        low = matches[i]
        assert high > low, ('CHANGES.md version order mismatch: %r <= %r' %
                            (high, low))
    # Double check that the last version equals the version in the pstorelib,
    # if pstorelib is installed already.
    if VERSION_STRING:
        assert matches[0] == VERSION_STRING, ('pstorelib version does not '
                                              'match CHANGES.md')
    # Fetch "current" version
    version = str(matches[0])

with open('README.txt') as file:
    long_description = file.read()

defaults = {
    'version': version,
    'author': 'Walter Doekes',
    'author_email': 'wjdoekes+pstore@osso.nl',
    'url': 'https://github.com/ossobv/pstore#jump',
    'license': 'LGPLv3',
    'long_description': long_description,
    'classifiers': [
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Framework :: Django',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: System Administrators',
        ('License :: OSI Approved :: GNU Lesser General Public License v3 or '
         'later (LGPLv3+)'),
        'Natural Language :: English',
        'Operating System :: POSIX',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Security',
        'Topic :: System :: Archiving',
        'Topic :: System :: Systems Administration',
    ],
}

EXCLUDE_FILES = (
    # These contain the local settings. We include the settings.py.template
    # instead.
    ('pstore', 'settings', 'pstore/settings.py'),
)


def setup_pstore():
    setup(
        name='pstore',
        packages=['pstorelib'],
        package_data={'pstorelib': [
            'lgpl-3.0.txt',
        ]},
        scripts=[
            'bin/pstore',   # no .py suffix for executables
        ],
        install_requires=['pygpgme', 'pyasn1>=0.0.8a', 'pycrypto>=2.0.1'],

        description='Python Protected Password Store library and client',
        keywords='password encrypted sharing cli',
        **defaults
    )


def setup_django_pstore():
    # Assert that our custom module finder is used.
    global EXCLUDE_FILES_HACK
    EXCLUDE_FILES_HACK = False

    setup(
        name='django-pstore',
        packages=['pstore'],
        package_data={'pstore': [
            'lgpl-3.0.txt',
            '*.template',
            'fixtures/*',
            'templates/*.html',
        ]},
        install_requires=['Django>=1.3,<1.5', 'pstore'],

        description='Python Protected Password Store server application',
        keywords='password encrypted sharing cli',
        **defaults
    )

    if not EXCLUDE_FILES_HACK:
        import sys
        if sys.argv == ['setup.py', 'register']:
            pass
        else:
            raise RuntimeError('Included local settings! Aborting!')


def find_package_modules(self, package, package_dir):
    """
    Hack to overcome deficiency in distutils/setuptools -- inability to
    exclude specific files. Modify the build process to exclude specific
    files from the build. Adapted to assert that the hack still works.

    Original: http://xylld.wordpress.com/2009/09/24/
                python-setuptools-workaround-for-ignore-specific-files/
    """
    # Make a note that this still works.
    global EXCLUDE_FILES_HACK
    EXCLUDE_FILES_HACK = True

    modules = _find_package_modules_orig(self, package, package_dir)
    for pkg, module, fname in EXCLUDE_FILES:
        if (pkg, module, fname) in modules:
            modules.remove((pkg, module, fname))
            print('excluding pkg = %s, module = %s, fname = %s' % (
                (pkg, module, fname)))
    return modules

build_py.find_package_modules = find_package_modules


# Hack to allow two setup() calls in the same setup.py. When called from the
# `make dist` Makefile command this bit below is replaced by either a call to
# `setup_pstore` or to `setup_django_pstore`.
setup_pstore()
